#include <rum/http/uri.h>
#include <exception>
#include <iostream>
#include <regex>
#include <sstream>
#if __cplusplus >= 202302L
#include <format>
#else
#include <sstream>
#endif
#include <string>

namespace Rum::HTTP {

URI::operator std::string() const {
  std::string query;
  for (auto param : parameters) {
    query += "\t\t'" + param.first + "': '" + param.second + "'\n";
  }
#if __cplusplus < 202302L
  std::stringstream text;
  text << "URI{{"
       << "\n\tscheme:   '" << scheme << "'"
       << "\n\tuser:     '" << user << "'"
       << "\n\tpassword: '" << password << "'"
       << "\n\thost:     '" << host << "'"
       << "\n\tport:     '" << port << "'"
       << "\n\tpath:     '" << path << "'"
       << "\n\tquery:   \n"
       << query << "\n\tfragment: '" << fragment << "'\n}}";

  return text.str();
#else
  return std::vformat(
      "URI{{"
      "\n\tscheme:   '{}'"
      "\n\tuser:     '{}'"
      "\n\tpassword: '{}'"
      "\n\thost:     '{}'"
      "\n\tport:     '{}'"
      "\n\tpath:     '{}'"
      "\n\tquery:   \n{}"
      "\n\tfragment: '{}'"
      "\n}}",
      std::make_format_args(scheme, user, password, host, port, path, query, fragment));
#endif
}

URI::URI(const std::string& uri) {
  std::regex uri_regex(R"((([\w\d]+):\/\/)?(([\w\d%]+)(:([\w\d%]+)?)@)?([\w\d\.]+)(:(\d+))?(\/?[\w\d%\.\/]+)?(\?([^#]+))?(\#([\w\d%]*))?)");

  std::smatch match;

#ifdef DEBUG
  std::cout << uri_decoded << std::endl;
#endif

  if (std::regex_match(uri.cbegin(), uri.cend(), match, uri_regex)) {
#define MATCH(var, i)   \
  if (match[i].matched) \
    var = match.str(i);

    MATCH(scheme, 2)
    MATCH(user, 4)
    user = decode(user);
    MATCH(password, 6)
    password = decode(password);
    MATCH(host, 7)
    MATCH(port, 9)
    MATCH(path, 10)
    path = decode(path);
    std::string query;
    MATCH(query, 12)
    query = decode(query);
    MATCH(fragment, 14)
    fragment = decode(fragment);

    std::stringstream parameters_string(query);
    std::string parameter;
    while (std::getline(parameters_string, parameter, '&')) {
      if (size_t pos = parameter.find('='); pos != parameter.npos) {
        parameters[parameter.substr(0, pos)] = parameter.substr(pos + 1);
      } else {
        parameters[parameter] = "";
      }
    }
  } else {
    throw std::exception();
  }
}

std::string URI::decode(std::string text) {
  std::regex encoded_regex(R"(%([\w\d]{2}))");
  std::smatch match;
  while (std::regex_search(text.cbegin(), text.cend(), match, encoded_regex)) {
    char decoded = (char)std::stoi(match.str(1), 0, 16);
    text = text.substr(0, match.position()) + decoded + text.substr(match.position() + 3);
  }

  return text;
}
std::string URI::encode(std::string text) {
  throw std::exception();
  return text;
}
} // namespace Rum::HTTP